<?php

/** ----------------------------------------------------------------------------
 *
 * datatables.php
 *
 * WEB-Shop Software http://www.webs.de/
 *
 * Helper class for the usage of jQuery DataTables.
 * http://www.datatables.net/
 *
 * -----------------------------------------------------------------------------
 * Third Party contributions:
 *  http://www.datatables.net/release-datatables/examples/data_sources/server_side.html
 * 
 * Released under the GNU General Public License
 *
 * @author Mike Küster <mike@xantiva.de>
 * @version $Id: DataTables.php 2247 2012-04-06 19:42:32Z xantiva $
 * @copyright 2012 WEB-Shop Software http://www.webs.de/
 *
 * -----------------------------------------------------------------------------
 */
require_once(DIR_FS_INC . 'xtc_db_query.inc.php');
require_once(DIR_FS_INC . 'xtc_db_input.inc.php');
require_once(DIR_FS_INC . 'xtc_db_fetch_array.inc.php');

class DataTables
{

    private $columns = array();
    private $echo = "";
    private $from = "";
    private $indexColumn = "*";
    private $limit = "";
    private $linkForItem = null;
    private $orderBy = "";
    private $result = array();
    private $where = "";
    private $useColumnForLink = null;

    /**
     * Constructor
     * 
     * @param array $get The $_GET variable.
     * @param array $columns Array with the names of the columns.
     * @param string $from FROM part of the SQL query.
     * @param string $where WHERE part of the SQL query.
     * @param string $indexColumn Column, which should be used as index. Default = "*".
     * @param string $linkForItem If the items should be links, the link href with a "%s" at the individual position.
     * @param string $useColumnForLink Which column contains the individual value for the link.
     */
    function __construct($get = array(), $columns = array(), $from = "", $where = "", $indexColumn = "*", $linkForItem = null, $useColumnForLink = null)
    {
        $this->columns = $columns;
        $this->from = $from;
        $this->where = $where;
        $this->indexColumn = $indexColumn;
        if (isset($linkForItem) && isset($useColumnForLink)) {
            $this->linkForItem = $linkForItem;
            $this->useColumnForLink = $useColumnForLink;
        }
        $this->parseGetParameter($get);
        $this->performQueries();
    }

    /**
     * Parse the parameter from the request
     * 
     * @param array $params The array (normally from $_GET) with the parameters
     * 						from the requerst.
     */
    private function parseGetParameter($params)
    {

        /*
         * echo number
         */
        $this->echo = intval($params['sEcho']);

        /*
         * Paging
         */
        if (isset($params['iDisplayStart']) && $params['iDisplayLength'] != '-1') {
            $this->limit = "LIMIT " . xtc_db_input($params['iDisplayStart']) . ", " .
                    xtc_db_input($params['iDisplayLength']);
        }

        /*
         * Ordering
         */
        if (isset($params['iSortCol_0'])) {
            $this->orderBy = "ORDER BY  ";
            for ($i = 0; $i < intval($params['iSortingCols']); $i++) {
                if ($params['bSortable_' . intval($params['iSortCol_' . $i])] == "true") {
                    $this->orderBy .= $this->columns[intval($params['iSortCol_' . $i])] . "
				 	" . xtc_db_input($params['sSortDir_' . $i]) . ", ";
                }
            }
            $this->orderBy = substr_replace($this->orderBy, "", -2);
            if ($this->orderBy == "ORDER BY") {
                $this->orderBy = "";
            }
        }

        /*
         * Filtering
         * NOTE this does not match the built-in DataTables filtering which does it
         * word by word on any field. It's possible to do here, but concerned about efficiency
         * on very large tables, and MySQL's regex functionality is very limited
         */
        $sWhere = "";
        if ($params['sSearch'] != "") {
            if ($this->where === "") {
                $sWhere = "WHERE (";
            } else {
                $sWhere = " AND (";
            }
            for ($i = 0; $i < count($this->columns); $i++) {
                $sWhere .= $this->columns[$i] . " LIKE '%" . xtc_db_input($params['sSearch']) . "%' OR ";
            }
            $sWhere = substr_replace($sWhere, "", -3);
            $sWhere .= ')';
        }

        /* Individual column filtering */
        for ($i = 0; $i < count($this->columns); $i++) {
            if ($params['bSearchable_' . $i] == "true" && $params['sSearch_' . $i] != '') {
                if ($sWhere == "") {
                    $sWhere = "WHERE ";
                } else {
                    $sWhere .= " AND ";
                }
                $sWhere .= $this->columns[$i] . " LIKE '%" . xtc_db_input($params['sSearch_' . $i]) . "%' ";
            }
        }
        $this->where .= $sWhere;
    }

    /**
     * Perform the needed SQL queries.
     */
    private function performQueries()
    {
        /*
         * SQL queries
         * Get data to display
         */
        $query = "
			SELECT SQL_CALC_FOUND_ROWS " . str_replace(" , ", " ", implode(", ", $this->columns)) . "
			$this->from
			$this->where
			$this->orderBy
			$this->limit
		";
        $rResult = xtc_db_query($query);

        /* Data set length after filtering */
        $query = "
			SELECT FOUND_ROWS()
		";
        $result = xtc_db_query($query);
        $row = xtc_db_fetch_array($result);
        $filteredTotal = $row['FOUND_ROWS()'];

        /* Total data set length */
        $query = "
			SELECT COUNT($this->indexColumn)
			$this->from
			$this->where
		";
        $result = xtc_db_query($query);
        $row = xtc_db_fetch_array($result);
        $total = $row["COUNT($this->indexColumn)"];


        /*
         * Output
         */
        $this->result = array(
            "sEcho" => $this->echo,
            "iTotalRecords" => $total,
            "iTotalDisplayRecords" => $filteredTotal,
            "aaData" => array()
        );

        if (isset($this->linkForItem)) {
            $format = "<a href='" . $this->linkForItem . "'>%s</a>";
        }

        while ($aRow = xtc_db_fetch_array($rResult)) {
            $row = array();
            $numberOfColumns = count($this->columns);
            for ($i = 0; $i < $numberOfColumns; $i++) {
                if (($this->columns[$i] != ' ') && ($this->columns[$i] != $this->useColumnForLink)) {
                    if (isset($this->linkForItem)) {
                        $row[] = sprintf($format, $aRow[$this->useColumnForLink], $aRow[$this->columns[$i]]);
                    } else {
                        $row[] = $aRow[$this->columns[$i]];
                    }
                }
            }
            $this->result['aaData'][] = $row;
        }
    }

    /**
     * Returns the JSON encoded result for the jQuery DataTables.
     * 
     * @return string  
     */
    public function getJSON()
    {
        return json_encode($this->result);
    }

}